import React, { useEffect, useRef, useState } from 'react'
import { Box, Grid } from '@mui/material'
import { useTheme, type Theme } from '@mui/material/styles'
import useMediaQuery from '@mui/material/useMediaQuery'
import {
  LocalVideoTrack,
  RemoteUser,
  useLocalCameraTrack,
  useLocalMicrophoneTrack,
  useRemoteUsers,
  usePublish,
} from 'agora-rtc-react'

const MeetingGrid = () => {
  const theme: Theme = useTheme()
  const isSm = useMediaQuery(theme.breakpoints.up('sm'))
  const isMd = useMediaQuery(theme.breakpoints.up('md'))
  const isLg = useMediaQuery(theme.breakpoints.up('lg'))

  const { localCameraTrack } = useLocalCameraTrack()
  const { localMicrophoneTrack } = useLocalMicrophoneTrack()
  usePublish([localMicrophoneTrack, localCameraTrack])
  const remoteUsers = useRemoteUsers()

  const gridContainerRef = useRef<HTMLDivElement>(null)
  const [gridContainerHeight, setGridContainerHeight] = useState(0)

  const [totalUsers, setTotalUsers] = useState(20)
  const [itemOffset, setItemOffset] = useState(12)
  const [itemHeight, setItemHeight] = useState(230)
  const [numOfItems, setNumOfItems] = useState(1)

  const maxUsersDisplay = isLg ? 12 : isMd ? 9 : isSm ? 4 : 2

  useEffect(() => {
    setTotalUsers(remoteUsers.length + 1)
  }, [remoteUsers])

  useEffect(() => {
    setNumOfItems(totalUsers > maxUsersDisplay ? maxUsersDisplay : totalUsers)
  }, [totalUsers])

  useEffect(() => {
    if (gridContainerRef.current) {
      setGridContainerHeight(gridContainerRef.current.offsetHeight)
    }
  }, [gridContainerRef?.current?.offsetHeight])

  useEffect(() => {
    switch (numOfItems) {
      case 1:
        setItemOffset(12)
        setItemHeight(gridContainerHeight)
        break
      case 2:
        setItemOffset(isSm ? 6 : 12)
        setItemHeight(isSm ? gridContainerHeight : (gridContainerHeight - 16) / 2)
        break
      case 3:
        setItemOffset(isMd ? 4 : isSm ? 6 : 12)
        setItemHeight(
          isMd
            ? gridContainerHeight
            : isSm
              ? (gridContainerHeight - 16) / 2
              : (gridContainerHeight - 32) / 3,
        )
        break
      case 4:
        setItemOffset(isSm ? 6 : 12)
        setItemHeight(isSm ? (gridContainerHeight - 16) / 2 : (gridContainerHeight - 16 * 3) / 4)
        break
      case 5:
        setItemOffset(isMd ? 4 : isSm ? 6 : 12)
        setItemHeight(
          isMd
            ? (gridContainerHeight - 16) / 2
            : isSm
              ? (gridContainerHeight - 16 * 2) / 3
              : (gridContainerHeight - 32 * 4) / 5,
        )
        break
      case 6:
        setItemOffset(isMd ? 4 : isSm ? 6 : 12)
        setItemHeight(
          isMd
            ? (gridContainerHeight - 16) / 2
            : isSm
              ? (gridContainerHeight - 16 * 2) / 3
              : (gridContainerHeight - 32 * 5) / 6,
        )
        break
      case 7:
        setItemOffset(isLg ? 3 : isMd ? 4 : isSm ? 6 : 12)
        setItemHeight(
          isLg
            ? (gridContainerHeight - 16) / 2
            : isMd
              ? (gridContainerHeight - 16 * 2) / 3
              : isSm
                ? (gridContainerHeight - 16 * 3) / 4
                : (gridContainerHeight - 32 * 6) / 7,
        )
        break
      case 8:
        setItemOffset(isLg ? 3 : isMd ? 4 : isSm ? 6 : 12)
        setItemHeight(
          isLg
            ? (gridContainerHeight - 16) / 2
            : isMd
              ? (gridContainerHeight - 16 * 2) / 3
              : isSm
                ? (gridContainerHeight - 16 * 3) / 4
                : (gridContainerHeight - 32 * 7) / 8,
        )
        break
      case 9:
        setItemOffset(isMd ? 4 : isSm ? 6 : 12)
        setItemHeight(
          isMd
            ? (gridContainerHeight - 16 * 2) / 3
            : isSm
              ? (gridContainerHeight - 16 * 4) / 5
              : (gridContainerHeight - 32 * 8) / 9,
        )
        break
      default:
        setItemOffset(isLg ? 3 : isMd ? 4 : isSm ? 6 : 12)
        setItemHeight(
          isLg ? (gridContainerHeight - 16 * 2) / 3 : (gridContainerHeight - 16 * 3) / 4,
        )
    }
  }, [totalUsers, isSm, isMd, isLg, gridContainerHeight, numOfItems])

  //   const onUserPublish = async (user: IAgoraRTCRemoteUser, mediaType: 'video' | 'audio') => {
  //     if (mediaType === 'video') {
  //       const remoteTrack = await agoraEngine.subscribe(user, mediaType)
  //       remoteTrack.play('remote-video')
  //     }
  //     if (mediaType === 'audio') {
  //       const remoteTrack = await agoraEngine.subscribe(user, mediaType)
  //       remoteTrack.play()
  //     }
  //   }

  return (
    <Box
      sx={{
        width: '100%',
        height: '80%',
        backgroundColor: theme.palette.background.default,
        padding: '30px',
        display: 'flex',
        flexDirection: 'column',
      }}
    >
      <Box
        sx={{
          display: 'flex',
          flexDirection: 'row',
          width: '100%',
          height: '100%',
        }}
        ref={gridContainerRef}
      >
        <Box sx={{ display: 'flex', width: '100%' }}>
          <Grid container spacing={3} width={'100%'} padding={0}>
            <Grid item xs={itemOffset}>
              <Box
                sx={{
                  backgroundColor: '#262625',
                  height: `${itemHeight}px`,
                  borderRadius: '15px',
                }}
              >
                <LocalVideoTrack
                  track={localCameraTrack}
                  play={true}
                  style={{ width: '100%', height: '100%', borderRadius: '20%' }}
                />
              </Box>
            </Grid>
            {remoteUsers.map((remoteUser, index) => (
              <Grid item xs={itemOffset} key={index}>
                <Box
                  sx={{
                    backgroundColor: '#262625',
                    height: `${itemHeight}px`,
                    borderRadius: '15px',
                  }}
                >
                  <RemoteUser
                    user={remoteUser}
                    playVideo={true}
                    playAudio={true}
                    style={{ width: '100%', height: '100%', borderRadius: '15px' }}
                  />
                </Box>
              </Grid>
            ))}
          </Grid>
        </Box>
      </Box>
    </Box>
  )
}

export default MeetingGrid
